@@ -14,7 +14,7 @@ Authen::HTTP::Signature::Method::HMAC - Compute digest using a symmetric key
 
 =cut
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 =head1 PURPOSE
 
@@ -81,7 +81,7 @@ sub _pad_base64 {
     my $n = length($b64_str) % 4;
 
     if ( $n ) {
-        $b64_str .= '=' x $n;
+        $b64_str .= '=' x (4-$n);
     }
 
     return $b64_str;
@@ -94,16 +94,14 @@ sub _get_digest {
     my $key = shift;
 
     my $digest;
-    for ( $algo ) {
-        when ( /sha1/ ) {
-            $digest = hmac_sha1_base64($data, $key);
-        }
-        when ( /sha256/ ) {
-            $digest = hmac_sha256_base64($data, $key);
-        }
-        when ( /sha512/ ) {
-            $digest = hmac_sha512_base64($data, $key);
-        }
+    if ( $algo =~ /sha1/ ) {
+        $digest = hmac_sha1_base64($data, $key);
+    }
+    elsif ( $algo =~ /sha256/ ) {
+        $digest = hmac_sha256_base64($data, $key);
+    }
+    elsif ( $algo =~ /sha512/ ) {
+        $digest = hmac_sha512_base64($data, $key);
     }
 
     confess "I couldn't get a $algo digest\n" unless defined $digest && length $digest;
